#!/python/bin

"""
Script that will create a new TMC file
This version does not work well for WFPC2, or any of the legacy
instruments
"""

import astropy.io.fits as py
import calendar
import time
import fits_functions as ff
import sys
import os
import argparse
import glob
import numpy as np


def UseafterToday():
        '''
        Takes the current day ad uses it to fill the useafter_var
        '''
        months=dict((k,v) for k,v in enumerate(calendar.month_abbr))
        localtime = time.localtime(time.time())
        return  months[localtime.tm_mon] + ' ' + \
                str(localtime.tm_mday) + ' ' + str(localtime.tm_year) + ' ' + \
                str(localtime.tm_hour) + ':' + \
                str(localtime.tm_min)  + ':' +  str(localtime.tm_sec)

#list of cos component names for MJD
cosmjdcomps=['cosmcp_g130mc1055m','cosmcp_g130mc1096m','cosmcp_g130mc1222m', \
              'cosmcp_g130mc1291m','cosmcp_g130mc1300m','cosmcp_g130mc1309m', \
              'cosmcp_g130mc1318m','cosmcp_g130mc1327m','cosmcp_g140lc1230m', \
              'cosmcp_g140lc1280m','cosmcp_g160mc1577m','cosmcp_g160mc1589m', \
              'cosmcp_g160mc1600m','cosmcp_g160mc1611m','cosmcp_g160mc1623m', \
              'cosncm3g185mc1786m','cosncm3g185mc1817m','cosncm3g185mc1835m', \
              'cosncm3g185mc1850m','cosncm3g185mc1864m','cosncm3g185mc1882m',\
              'cosncm3g185mc1890m','cosncm3g185mc1900m','cosncm3g185mc1913m',\
              'cosncm3g185mc1921m','cosncm3g185mc1941m','cosncm3g185mc1953m',\
              'cosncm3g185mc1971m','cosncm3g185mc1986m','cosncm3g185mc2010m',\
              'cosncm3g225mc2186m','cosncm3g225mc2217m','cosncm3g225mc2233m',\
              'cosncm3g225mc2250m','cosncm3g225mc2268m','cosncm3g225mc2283m',\
              'cosncm3g225mc2306m','cosncm3g225mc2325m','cosncm3g225mc2339m',\
              'cosncm3g225mc2357m','cosncm3g225mc2373m','cosncm3g225mc2390m',\
              'cosncm3g225mc2410m','cosncm3g230lc2635m','cosncm3g230lc2950m',\
              'cosncm3g230lc3000m','cosncm3g230lc3360m','cosncm3g230lc3360m',\
              'cosncm3g285mc2617m','cosncm3g285mc2637m','cosncm3g285mc2657m',\
              'cosncm3g285mc2676m','cosncm3g285mc2695m','cosncm3g285mc2709m',\
              'cosncm3g285mc2719m','cosncm3g285mc2739m','cosncm3g285mc2850m',\
              'cosncm3g285mc2952m','cosncm3g285mc2979m','cosncm3g285mc2996m',\
              'cosncm3g285mc3018m','cosncm3g285mc3035m','cosncm3g285mc3057m',\
              'cosncm3g285mc3074m','cosncm3g285mc3094m']

# Pysynphot directory path for each instrument
path_instr={'acs' : 'cracscomp$', 'cos' : 'crcoscomp$', 'stis' :
        'crstiscomp$','wfc3' :'crwfc3comp$', 'wfpc2' : 'crwfpc2comp$',
        'nonhst' : 'crnonhstcomp$', 'foc' : 'crfoccomp$', 'hrs' : \
        'crhrscomp$', 'hsp' : 'crhspcomp$', 'nicmos' : 'crnicmoscomp$', \
        'wfpc' : 'crwfpccomp$'}

# ACS parameterized keyword
acs_fr={'acs_fr388n' : '[fr388n#]', 'acs_fr388n' : '[fr388n#]', \
        'acs_fr423n' : '[fr423n#]', 'acs_fr459m' : '[fr459m#]', \
        'acs_fr462n' : '[fr462n#]', 'acs_fr505n' : '[fr505n#]', \
        'acs_fr551n' : '[fr551n#]', 'acs_fr601n' : '[fr601n#]', \
        'acs_fr647m' : '[fr647m#]', 'acs_fr656n' : '[fr656n#]', \
        'acs_fr716n' : '[fr716n#]', 'acs_fr782n' : '[fr782n#]', \
        'acs_fr853n' : '[fr853n#]', 'acs_fr914m' : '[fr914m#]'}

# WFPC2 parameterized keyword cont
wfpc2_cont =['wfpc2_contpc1','cont wfpc2_contwf2', 'wfpc2_contwf3','wfpc2_contwf4']


def parse_args():

    '''
    Parse line arguments
    '''

    parser = argparse.ArgumentParser()
    parser.add_argument('-f','--tmcinput',action='store',dest='tmcin')
    parser.add_argument('-a','--author',action='store',dest='author')
    parser.add_argument('-r','--reason',action='store',dest='reason')
    args = parser.parse_args()
    return args


if __name__ == "__main__":
    args = parse_args()

    if args.tmcin  == None:
         print('You need to provide the name of the input TMC file')
         sys.exit(0)

    #Look at all the files in the working directory
    files = glob.glob('*_syn.fits')

    #Reads and save the parameters that will be used to update/modify the TMC
    file
    new_files=[]
    components=[]
    instruments=[]
    descrip=[]
    for f in files:
        components.append(ff.get_keyword(f,'COMPNAME',0))
        instruments.append(ff.get_keyword(f,'INSTRUME',0))
        descrip.append(ff.get_keyword(f,'DESCRIP',0))
        new_files.append(f)

    # looks if the files being delivered have unique compnames
    uniq = []
    repeated = set()
    for i in xrange(len(components)):
       if components[i] not in repeated:
          uniq.append(components[i])
          repeated.add(components[i])
       else:
          print 'repeated compname ', components[i],' in file ',new_files[i]
          print 'abort'
          sys.exit(1)

    # define output filename
    outfile = 'new_upd_tmc.fits'
    if os.access(outfile, os.R_OK):
        os.remove(outfile)

    # Define primary heder and add the needed keywords
    newhdu= py.PrimaryHDU()
    newhdu.header['USEAFTER'] = UseafterToday()
    newhdu.header['INSTRUME'] = 'HST'
    newhdu.header['DBTALE'] = 'CRCOMPLIST'
    newhdu.header['DESCRIP'] = 'The lookup table for the HST  component throughputs'
    if args.author == None:
        newhdu.header['COMMENT'] = '= Reference file created by R.Diaz'
    else:
        newhdu.header['COMMENT'] = '= Reference file created by ' + args.author

    today_time=UseafterToday()

    #Uses input FITS file to  set things up
    filein = py.open(args.tmcin)   # open file
    prihdr = filein[0].header

    # Copies the history from the latest file and adds new input if a reason is
    # given
    for  l in xrange(len(prihdr)-15):
        newhdu.header.add_history(prihdr[l+13])
    newhdu.header.add_history(' ')
    newhdu.header.add_history('Updated on ' + UseafterToday())
    newhdu.header.add_history(' ')
    if args.reason != None:
       newhdu.header.add_history(args.reason)

    #Reads the input from the old TMC file
    data_tmc = filein[1].data     # read data
    cn=data_tmc.field('compname')
    fn=data_tmc.field('filename')
    tn=data_tmc.field('time')
    cmm= data_tmc.field('comment')

    #copy them to arrays to be able to append
    time_created=np.array(tn)
    compname=np.array(cn)
    filename=np.array(fn)
    comment=np.array(cmm)

    # If it finds new components it saves them
    new_component=[]
    new_time=[]
    new_filename=[]
    new_compname=[]
    new_comment=[]
    # Finds the row that has the component for the file we are delivering
    for i in xrange(len(components)):
        update = 'no'
        for  j in xrange(len(compname)):
            if components[i] == compname[j]:
                 update = 'yes'
                 time_created[j] = today_time
                 if str.lower(instruments[i]) == 'acs':
                     aperf=components[i].find('_aper',0,len(components[i]))
                     if acs_fr.has_key(compname[j]) == True:
                         filename[j]= path_instr['acs'] + new_files[i] + \
                                 acs_fr[compname[j]]
                     elif aperf != -1:
                             filename[j]= path_instr[str.lower(instruments[i])] +\
                                new_files[i] + '[aper#]'
                     else:
                         filename[j]= path_instr['acs'] + new_files[i]
                 elif str.lower(instruments[i]) == 'cos':
                    for k in xrange(len(cosmjdcomps)):
                        if compname[j] == cosmjdcomps[k]:
                            filename[j]= path_instr['cos'] + new_files[i] + '[mjd#]'
                            break
                        else:
                            filename[j]= path_instr['cos'] + new_files[i]
                 elif str.lower(instruments[i]) == 'wfc3':
                     aperf=components[i].find('_aper',0,len(components[i]))
                     mjdf=components[i].find('_m',0,len(components[i]))
                     if mjdf != -1:
                         filename[j]= path_instr['wfc3'] + new_files[i] + '[mjd#]'
                     elif aperf != -1:
                             filename[j]= path_instr[str.lower(instruments[i])] +\
                                new_files[i] + '[aper#]'
                     else:
                         filename[j]= path_instr['wfc3'] + new_files[i]
                 elif str.lower(instruments[i]) == 'wfpc2':
                    for k in xrange(len(wfpc2_cont)):
                        if compname[j] == wfpc2_cont[k]:
                            filename[j]= path_instr['wfpc2'] + new_files[i] + '[cont#]'
                            break
                        else:
                         filename[j]= path_instr['wfpc2'] + new_files[i]
                 else:
                    mjdf=components[i].find('_mjd',0,len(components[i]))
                    aperf=components[i].find('_aper',0,len(components[i]))
                    if mjdf != -1:
                        filename[j]= path_instr[str.lower(instruments[i])] + \
                                new_files[i] + '[mjd#]'
                    elif aperf != -1:
                        filename[j]= path_instr[str.lower(instruments[i])] +\
                                new_files[i] + '[aper#]'
                    else:
                        filename[j]= path_instr[str.lower(instruments[i])] +\
                                new_files[i]
        if update == 'no':
            print 'New component ',components[i], ' adding it to the TMC file'
            new_time.append(today_time)
            new_compname.append(components[i])
            newf=path_instr[instruments[i]] + new_files[i]
            new_filename.append(newf)
            new_comment.append(descrip[i])

    if len(new_time) > 0:
        for i in xrange(len(new_time)):
            time_created=np.append(time_created,new_time[i])
            compname=np.append(compname,new_compname[i])
            filename=np.append(filename,new_filename[i])
            comment=np.append(comment,new_comment[i])

    oldt=min(time_created)
    oldest=time.strptime(oldt, "%b %d %Y %H:%M:%S")
    newt=max(time_created)
    newest=time.strptime(newt, "%b %d %Y %H:%M:%S")
    print oldest
    print newest
    print oldest.tm_year
    pedigre0='INFLIGHT ' + str(oldest.tm_mon)+ '/' + str(oldest.tm_mday) \
            + '/' + str(oldest.tm_year)
    #+ ' ' + newest.month + '/' + newest.day + '/' + \
            #        newest.year
    newhdu.header.update('pedigree', pedigre0)

    # looks if the files being delivered have unique compnames
    uniq = []
    repeated = set()
    print ('\n Checking that there are no repeated components ..... \n')
    for i in xrange(len(compname)):
       if compname[i] not in repeated:
          uniq.append(compname[i])
          repeated.add(compname[i])
       else:
          print 'repeated compname ', compname[i]
          print 'abort'
          sys.exit(1)

    ind=np.argsort((compname))
    # Defines columns and asign their values
    col1= py.Column(name='TIME', format='26A', array=time_created[ind],
        disp='26A')
    col2= py.Column(name='COMPNAME', format='18A', array=compname[ind],
        disp='18A')
    col3= py.Column(name='FILENAME', format='56A', array=filename[ind],
        disp='50A')
    col4= py.Column(name='COMMENT', format='68A', array=comment[ind],
        disp='68A')

    cols=py.ColDefs([col1,col2,col3,col4])
    tbhdu = py.BinTableHDU.from_columns(cols)

    thdulist = py.HDUList([newhdu,tbhdu])
    thdulist.writeto(outfile)

    print('new_upd_tmc.fits file was created')

